#!/usr/bin/env python3

#******************************************************************************
# treenodelist.py, provides a class to do operations on groups of nodes
#
# TreeLine, an information storage program
# Copyright (C) 2017, Douglas W. Bell
#
# This is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License, either Version 2 or any later
# version.  This program is distributed in the hope that it will be useful,
# but WITTHOUT ANY WARRANTY.  See the included LICENSE file for details.
#******************************************************************************

from PyQt5.QtGui import QClipboard
from PyQt5.QtWidgets import (QApplication, QMessageBox)
import undo


class TreeNodeList(list):
    """Class to do operations on groups of nodes.
    
    Stores a list of nodes.
    """
    def __init__(self, nodeList=None):
        """Initialize a tree node group.

        Arguments:
            nodeList -- the initial list of nodes
        """
        super().__init__()
        if nodeList:
            self[:] = nodeList

    def copyTree(self):
        """Copy these node branches to the clipboard.
        """
        if not self:
            return
        clip = QApplication.clipboard()
        if clip.supportsSelection():
            titleList = []
            for node in self:
                titleList.extend(node.exportTitleText())
            clip.setText('\n'.join(titleList), QClipboard.Selection)
        clip.setMimeData(self[0].modelRef.mimeData([node.index() for node in
                                                    self]))

    def pasteMimeData(self, mimeData):
        """Decode mime data and paste into these nodes.
        
        Returns True on success.
        Arguments:
            mimeData - the data to paste.
        """
        if not self:
            return False
        modelRef = self[0].modelRef
        undoBranches = self[:]
        undoObj = undo.BranchFormatUndo(modelRef.undoList, undoBranches,
                                        modelRef.formats)
        for parent in self:
            nodeList = modelRef.nodesFromMimeData(mimeData)
            if not nodeList:
                modelRef.undoList.removeLastUndo(undoObj)
                return False
            modelRef.addNodesToModel(nodeList, parent)
            for clone in parent.cloneGen():
                newNodeList = [node.newBranchClone() for node in nodeList]
                modelRef.addNodesToModel(newNodeList, clone, setId=False)
        return True

    def cloneMimeData(self, mimeData):
        """Decode mime data and paste linked clones into these nodes.

        Returns True on success.
        Arguments:
            mimeData - the data to paste.
        """
        if not self:
            return False
        modelRef = self[0].modelRef
        newNodeList = modelRef.nodesFromMimeData(mimeData)
        if not newNodeList:
            return False
        try:
            origNodeList = [modelRef.nodeIdDict[node.uniqueId] for node in
                            newNodeList]
            for node in newNodeList:
                for child in node.descendantGen():
                    modelRef.nodeIdDict[child.uniqueId]  # verify all here
        except KeyError:
            QMessageBox.warning(QApplication.activeWindow(), 'TreeLine',
                                _('Cannot paste clones from a separate file'))
            return False
        if sum([len(parent.clones) for parent in self]) > 0:
            QMessageBox.warning(QApplication.activeWindow(), 'TreeLine',
                                _('Cannot add a new clone to a cloned parent'))
            return False
        undo.BranchFormatUndo(modelRef.undoList, self + origNodeList,
                              modelRef.formats)
        for parent in self:
            nodeList = modelRef.nodesFromMimeData(mimeData)
            for node in nodeList:
                parent.childList.append(node)
                node.parent = parent
                for child in node.descendantGen():
                    child.modelRef = modelRef
                    origChild = modelRef.nodeIdDict[child.uniqueId]
                    child.clones.add(origChild.uniqueId)
                    child.clones.update(origChild.clones)
                    child.setUniqueId(True)
                    origChild.clones.add(child.uniqueId)
                    child.data = origChild.data
        return True
